/* 
 * CellInfo.java
 *
 * Copyright 2003, Leigh L. Klotz, Jr..  All Rights Reserved.
 *   Email: klotz@graflex.org
 *     Web: http: //www.graflex.org/klotz
 *
 * {LICENSE}
 */
package com.hipme.cellinfo;

import danger.app.*;
import danger.audio.Meta;
import danger.ui.*;
import danger.net.*;
import danger.util.*;
import danger.util.format.StringFormat;
import danger.util.DEBUG;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

class MainWindow extends ScreenWindow implements Resources, Events {
  final static int BINGO_THRESHOLD_ASU = 31;
  final static int HOT_THRESHOLD_ASU = 28;
  CellInfo app;
  CellListView listView;
  boolean showAllTowers = false;
  int seqNo = 0;

  MainWindow(CellInfo app) {
    this.app = app;
    Rect bounds = getBounds();
    int width = (bounds.right - bounds.left);
    int height = (bounds.bottom - bounds.top);
	    
    ResourceDatabase db = app.getResources();
    Menu menu = getActionMenu();
    menu.removeAllItems();
    menu.addFromResource(db, kCellInfoMenu);

    listView = new CellListView();
    listView.setSize(width, height);
    listView.setPosition(0, 0);
    addChild(listView);
    showAllChildren();
    setFocusedChild(listView);
    sendUpdate();
    DEBUG.p("cellinfo: mainwindow done");
  }
	
  void sendUpdate() {
    new Event(this, kUpdateList).sendToTarget();
  }

  // now we need locking...
  private void update() {
    DEBUG.p("cellinfo: update");
    DataStore ds = app.getDataStore();
    if (ds == null) return;
    int nRecords = ds.getRecordCount();
    //    int oldRecordID = 0;	// is 0 an OK out-of-range value?
    //    // Try to preserve the previously selected item through removeAllItems()
    //    // which we should do anyway.
    //    if (nRecords != 0) {
    //      try {
    //	MenuItem item = listView.getSelectedItem();
    //	oldRecordID = ((Data)item.getEventArg()).getRecordUID();
    //      } catch (Exception ex) { DEBUG.p(ex);ex.printStackTrace(); }
    //    }
    listView.removeAllItems();
    Vector v = Record.getCellInfo();
    String oper = (String)v.elementAt(0);
    String lac = (String)v.elementAt(1);
    String cid = (String)v.elementAt(2);
    int towerNoSector = (cid == null) ? -1 : Record.cidToTowerNoSector(cid);

    for (int i = 0; i < nRecords; i++) {
      Record record = new Record(ds.getRecordData(i));
      boolean sameTower = (cid != null  && cid.length()>0 && record.sameTower(oper, lac, towerNoSector));
      if (!showAllTowers && !sameTower)
	continue;
      int recordUID = ds.getRecordUID(i);
      record.setRecordUID(recordUID);
      MenuItem m = listView.addItem(record.getTitle(), this, kShowEditEntry, recordUID, record);
      if (record.isTowerLocation()) {
	m.setMarked(true);
      }
      if (sameTower) {
	m.setChecked(true);
      }
      //      if (oldRecordID != 0 && record.getRecordUID() == oldRecordID) {
      //	DEBUG.p("setting selection to " + oldRecordID);
      //	listView.setSelection(m);
      //      }
    }
    if (nRecords > 0) 
      listView.setSelection(nRecords-1);
    listView.invalidate();
    DEBUG.p("cellinfo: update done");
  }

  public boolean receiveEvent(Event event) {
    boolean consumed = true;
    switch (event.type) {
    case kReportCellLocation:
      handleReportCellLocation(event);
      break;
    case kShowEditEntry:
      showEditEntry(event);
      break;
    case kShowMapEvent:
      showMap(event);
      break;
    case kDiscardEntry:
      handleDiscardEntry(event);
      break;
    case kUpdateList:
      update();
      break;
    default:
      consumed = super.receiveEvent(event);
    }
    return consumed;
  }

  public boolean eventWidgetDown(int i, Event event) {
    boolean consumed = true;
    switch(i) {
    case Event.DEVICE_BUTTON_BACK:
      // X exits app
      sendExitApp();
      break;
    default:
      consumed = super.eventWidgetDown(i, event);
    }
    return consumed;
  }

  public final boolean eventKeyUp(char i, Event event) {
    boolean consumed = false;
    if (i == 8) {
      showDiscardEntry();
      consumed = true;
    }
    if (! consumed) 
      consumed = super.eventKeyUp(i, event);
    return consumed;
  }

  void sendExitApp() {
    new Event(Application.getCurrentApp(), kReturnToLauncher).sendToTarget();
  }

  void help() {
    new AlertWindow("Just type in your location and press DONE.  You can edit or delete your entries later.").show();
  }
	
  void showAllTowers(Event event) {
    showAllTowers = !showAllTowers;
    MenuItem m = ((Menu)(event.argument)).getItemWithIDOrNull(kCellInfoMenuShowAllTowers);
    m.setChecked(showAllTowers);
    sendUpdate();
  }

  private void handleReportCellLocation(Event event) {
    MyDialogWindow dw = ((MyDialogWindow)(event.argument));
    Record record = new Record();
    record.parseDialogWindow(dw);
    try {
      HTTPConnection conn = new HTTPConnection();
      if (true) {
	String postString = record.getAddEditPost();
	DEBUG.p("posting " + postString);
	conn.post(app.getString(kPostURL),
		  "Content-Type: application/x-www-form-urlencoded",
		  postString,
		  HTTPConnection.RETURN_HEADERS,
		  seqNo++);
      }
    } catch (Exception ioe) {
      new AlertWindow("ioe : "+ioe.getMessage()).show();
      DEBUG.p("ioe : "+ioe.getMessage());
    }
    sendUpdate();
  }

  void handleDiscardEntry(Event event) {
    // it's too hard to pass the item from showDiscardEntry to discardEntry through the dialog
    // so we just look it up again and hope hope there's no way to move the cursor
    MenuItem item = listView.getSelectedItem();
    if (item == null) {
      Meta.play(Meta.DELETE_ITEM);
      return;
    }
    Record record = ((Record)(item.getEventArg()));
    String uid = record.getUID();
    int recordUID =  item.getEvent().data;
    deleteCellLocation(uid, recordUID);
  }

  void deleteCellLocation(String uid, int recordUID) {
    DEBUG.p("deleteCellLocation(uid="+uid+", recordUID="+recordUID);
    try {
      HTTPConnection conn = new HTTPConnection();
      if (true) {
	String postString = "uid="+uid+"&recordUID="+recordUID+"&delete=true";
	DEBUG.p("posting " + postString);
	conn.post(app.getString(kPostURL),
		  "Content-Type: application/x-www-form-urlencoded",
		  postString,
		  HTTPConnection.RETURN_HEADERS,
		  seqNo++);
      }
    } catch (Exception ioe) {
      new AlertWindow("ioe : "+ioe.getMessage()).show();
      DEBUG.p("ioe : "+ioe.getMessage());
    }
  }

  void handleResponse(HTTPTransaction t) {
    DEBUG.p("response = " + t.getString());
    Record record = new Record();
    record.parsePost(t.getString());
    DataStore ds = app.getDataStore();
    DEBUG.p("handleResponse record.hasRecordUID()="+record.hasRecordUID());
    if (record.hasRecordUID()) {
      int recordUID = record.getRecordUID();
      DEBUG.p("handleResponse recordUID="+recordUID);
      if (record.isDeleted()) {
	// we have to try removing twice because
	// it doesn't seem to work to remove 
	DEBUG.p("removing recordUID="+recordUID);
	ds.removeRecordByUID(recordUID);
	// records with the wrong sign
	// and the sign changes when the record is sync'd.
	DEBUG.p("removing recordUID="+(-recordUID));
	ds.removeRecordByUID(-recordUID);
      } 
      else {
	DEBUG.p("Setting recordUID="+recordUID);
	ds.setRecordDataByUID(recordUID, record.getBytes());
      }
    }
    else {
      int idx = ds.addRecord(record.getBytes());
      DEBUG.p("Added record recordUID="+ds.getRecordUID(idx));
    }
    sendUpdate();
  }

  void showCellInfo() {
    String uid = null;
    Record record = new Record();
    record.update();
    showCellInfo(record);
  }


  void showEditEntry(Event event) {
    Record record = ((Record)(event.argument));
    showCellInfo(record);
  }

  void showDiscardEntry() {
    MenuItem item = listView.getSelectedItem();
    if (item == null) {
      Meta.play(Meta.DELETE_ITEM);
      return;
    }
    Record record = ((Record)(item.getEventArg()));
    MyDialogWindow dw = ((MyDialogWindow)(app.getDialog(kCellInfoDiscardDialog, this)));
    dw.setStringValue(kCellInfoDiscardDialogText, record.getTitle());
    dw.show();
  }

  final static String[] towerStrings = { "Can't Happen", "South", "West or North West", "East or North East" };	// I18N

  void showCellInfo(Record record) {
    MyDialogWindow dw = ((MyDialogWindow)(app.getDialog(kCellInfoDialog, this)));
    if (record.hasUID()) 
      dw.setStringValue(kCellInfoUID, record.getUID());
    if (record.hasRecordUID()) 
      dw.setStringValue(kCellInfoRecordUID, ""+record.getRecordUID());
    dw.setStringValue(kCellInfoOper, record.getOPER());
    dw.setStringValue(kCellInfoLAC, record.getLAC());
    dw.setStringValue(kCellInfoCID, record.getCID());
    dw.setStringValue(kCellInfoSignalStrength, record.getSignalStrength());
    dw.setStringValue(kCellInfoASU, "" + record.getASU());
    dw.setStringValue(kCellInfoDirection, towerStrings [record.getSector()]);
    dw.setStringValue(kCellInfoName, record.getName());
    dw.setStringValue(kCellInfoAddress, record.getAddress());
    dw.setStringValue(kCellInfoCity, record.getCity());
    dw.setStringValue(kCellInfoState, record.getState());
    dw.setStringValue(kCellInfoZip, record.getZip());
    dw.setStringValue(kCellInfoLatitude, record.getLatitude());
    dw.setStringValue(kCellInfoLongitude, record.getLongitude());
    dw.setBooleanValue(kCellInfoIsTowerLocation, record.isTowerLocation());
    dw.setBooleanValue(kCellInfoIsIndoors, record.isIndoors());
    {
      dw.hideText(kCellInfoBingo);
      dw.hideText(kCellInfoHot);
      if (record.getASU() >= BINGO_THRESHOLD_ASU)
	dw.showText(kCellInfoBingo);
      else if (record.getASU() >= HOT_THRESHOLD_ASU)
	dw.showText(kCellInfoHot);
    }
    dw.show();
  }

  void showMapDialog() {
    MyDialogWindow dw = ((MyDialogWindow)(app.getDialog(kShowMapDialog, this)));
    dw.setStringValue(kShowMapDialogTerm, "restaurant");
    PopupMenu menu = (PopupMenu) dw.getDescendantWithID(kShowMapDialogLocation);
    Vector v = new Vector();
    int nRecords = listView.itemCount();
    Hashtable dups = new Hashtable();
    for (int i = 0; i < nRecords; i++) {
      MenuItem inItem = listView.getItem(i);
      if (inItem.isChecked()) {
	Record record = ((Record)(inItem.getEventArg()));
	String title = null;
	String address = record.getAddress();
	String city = record.getCity();
	String state = record.getState();
	String zip = record.getZip();
	if (address.length()>0 && city.length()>0 && state.length()>0) 
	  title = (address+","+city+","+state);
	else if (address.length()>0 && zip.length() > 0) 
	  title = address+" " + zip;
	else if (zip.length()>0)
	  title = zip;
	if (title != null && !dups.containsKey(title)) {
	  dups.put(title, title);
	  MenuItem newItem = new MenuItem(title);
	  if (record.isTowerLocation()) newItem.setMarked(true);
	  v.addElement(newItem);
	}
      }
    }
    if (v.size() > 0) {
      MenuItem[] items = new MenuItem[v.size()];
      int i = 0;
      for (Enumeration e = v.elements(); e.hasMoreElements(); ) {
	items[i++] = (MenuItem)(e.nextElement());
      }
      menu.addItems(items);
    }
    dw.show();
  }

  void showMap(Event event) {
    MyDialogWindow dw = ((MyDialogWindow)(event.argument));
    PopupMenu menu = (PopupMenu) dw.getDescendantWithID(kShowMapDialogLocation);
    MenuItem selected = menu.getItem(menu.getValue());
    String location = selected.getTitle();
    String term = dw.getStringValue(kShowMapDialogTerm);
    String url = StringFormat.fromRsrc(kGoogleLocationUrl, StringUtils.urlEncode(term), StringUtils.urlEncode(location));
    try {
      URL.gotoURL(url);
    } catch (URLException e) {
      DEBUG.p("cellinfo: " + e);
      e.printStackTrace();
      Meta.play(Meta.BEEP_ACTION_FAIL);
    }
  }

}

