package com.hipme.cellinfo;

import java.util.Vector;
import danger.util.StringTokenizer;
import danger.util.StringUtils;
import danger.app.DataRecord;
import danger.util.DEBUG;

public final class Record implements Resources {
  DataRecord dataRecord;
  public DataRecord getDataRecord() { return dataRecord; }
  public byte[] getBytes() { return dataRecord.mData; }

  private final static int kStringRow = 0;
  private final static int kIntRow = 1;
  private final static int kShortRow = 2;
  private final static int kNumRows = 3;

  // update Record() if adding more strings
  private final static int kStringOPERCol = 0;
  private final static int kStringLACCol = 1;
  private final static int kStringCIDCol = 2;
  private final static int kStringSignalStrengthCol = 3;
  private final static int kStringUIDCol = 4;
  private final static int kStringNameCol = 5;
  private final static int kStringAddressCol = 6;
  private final static int kStringCityCol = 7;
  private final static int kStringStateCol = 8;
  private final static int kStringZipCol = 9;
  private final static int kStringLatitudeCol = 10;
  private final static int kStringLongitudeCol = 11;
  static String[] stringKeys = { "oper", "lac", "cid", "signalStrength", "uid", "name", "address", "city", "state", "zip", "latitude", "longitude" };
  
  // update Record() if adding more ints
  private final static int kIntRecordUIDCol = 0;
  static String[] intKeys = { "recordUID" };
  
  // update Record() if adding more shorts
  private final static int kShortIsTowerLocationCol = 0;
  private final static int kShortIsIndoorsCol = 1;
  private final static int kShortDeleteCol = 2;
  private final static int kShortAsuCol = 3;
  private final static int kShortDBMCol = 4;
  private final static int kShortTowerDecimalCol = 5;
  static String[] shortKeys = { "isTowerLocation", "isIndoors", "delete", "asu", "dBm", "towerDecimal" };

  private void put(String key, String value) {
    if (value == null || value.length() == 0) {
      // DEBUG.p("skipping " + key+" because value="+value);
      return;
    }
    final char c = value.charAt(0);
    boolean numstart = (Character.isDigit(c) || c == '-');
    // DEBUG.p("key="+key+" value="+value+" numstart="+numstart);
    if (numstart) {
      int i = getKeyNum(intKeys, key);
      if (i != -1) {
	// DEBUG.p("Setting int " + key + "="+value);
	dataRecord.putInt(kIntRow, i, StringUtils.toInt(value));
	return;
      }
    }

    if (numstart || c == 't' || c == 'f') {
      int i = getKeyNum(shortKeys, key);
      if (i != -1) {
	if (numstart) {
	  // DEBUG.p("Setting short " + key + "="+value);
	  dataRecord.putShort(kShortRow, i, StringUtils.toInt(value));
	  return;
	} else {
	  if (value.equals("true")) {
	    // DEBUG.p("Setting boolean " + key + "="+1);
	    dataRecord.putShort(kShortRow, i, 1);
	    return;
	  }
	  else if (value.equals("false")) {
	    // DEBUG.p("Setting boolean " + key + "="+0);
	    dataRecord.putShort(kShortRow, i, 0);
	    return;
	  }
	}
      }
    }

    int i = getKeyNum(stringKeys, key);
    if (i != -1) {
      // DEBUG.p("Setting string " + key + "="+value);
      dataRecord.putString(kStringRow, i, value);
      return;
    }  

    throw new java.util.NoSuchElementException(key);
  }

  private boolean putInternal(String key, String value) {
    for (int i = 0; i < stringKeys.length; i++) {
      if (key.equals(stringKeys[i])) {
	// DEBUG.p("match " + i+" key="+key+" value="+value);
	dataRecord.putString(kStringRow, i, value);
	return true;
      }
    }
    return false;
  }

  private int getKeyNum(String[] keys, String key) {
    for (int i = 0; i < keys.length; i++) {
      if (key.equals(keys[i])) {
	return i;
      }
    }
    return -1;
  }

  String getString(String key) {
    int i = getKeyNum(stringKeys, key);
    if (i < 0)
      throw new java.util.NoSuchElementException(key);
    return dataRecord.getString(kStringRow, i);
  }

  String getOPER() { return dataRecord.getString(kStringRow, kStringOPERCol); }
  String getLAC() { return dataRecord.getString(kStringRow, kStringLACCol); }
  String getCID() { return dataRecord.getString(kStringRow, kStringCIDCol); }
  String getSignalStrength() { return dataRecord.getString(kStringRow, kStringSignalStrengthCol); }
  String getDBM() { return dataRecord.getString(kShortRow, kShortDBMCol); }
  int getASU() { return dataRecord.getShort(kShortRow, kShortAsuCol); }
  int getTowerDecimal() { return hexToDec(getCID()); }
  boolean hasUID() { return getUID().length()>0; }
  String getUID() { return dataRecord.getString(kStringRow, kStringUIDCol); }
  String getName() { return dataRecord.getString(kStringRow, kStringNameCol); }
  String getAddress() { return dataRecord.getString(kStringRow, kStringAddressCol); }
  String getCity() { return dataRecord.getString(kStringRow, kStringCityCol); }
  String getState() { return dataRecord.getString(kStringRow, kStringStateCol); }
  String getZip() { return dataRecord.getString(kStringRow, kStringZipCol); }
  String getLatitude() { return dataRecord.getString(kStringRow, kStringLatitudeCol); }
  String getLongitude() { return dataRecord.getString(kStringRow, kStringLongitudeCol); }
  boolean isTowerLocation() { return dataRecord.getShort(kShortRow, kShortIsTowerLocationCol) != 0; }
  boolean isIndoors() { return dataRecord.getShort(kShortRow, kShortIsIndoorsCol) != 0; }
  boolean isDeleted() { return dataRecord.getShort(kShortRow, kShortDeleteCol) != 0; }
  int getRecordUID() { return dataRecord.getInt(kIntRow, kIntRecordUIDCol); }
  boolean hasRecordUID() { return getRecordUID() !=0; }

  void setOPER(String value) { dataRecord.putString(kStringRow, kStringOPERCol, value); }
  void setLAC(String value) { dataRecord.putString(kStringRow, kStringLACCol, value); }
  void setCID(String value) { dataRecord.putString(kStringRow, kStringCIDCol, value); }
  void setSignalStrength(String value) { dataRecord.putString(kStringRow, kStringSignalStrengthCol, value); }
  void setDBM(String value) { dataRecord.putString(kStringRow, kShortDBMCol, value); }
  void setASU(short value) { dataRecord.putShort(kShortRow, kShortAsuCol, value); }
  void setUID(String value) { dataRecord.putString(kStringRow, kStringUIDCol, value); }
  void setName(String value) { dataRecord.putString(kStringRow, kStringNameCol, value); }
  void setAddress(String value) { dataRecord.putString(kStringRow, kStringAddressCol, value); }
  void setCity(String value) { dataRecord.putString(kStringRow, kStringCityCol, value); }
  void setState(String value) { dataRecord.putString(kStringRow, kStringStateCol, value); }
  void setZip(String value) { dataRecord.putString(kStringRow, kStringZipCol, value); }
  void setLatitude(String value) { dataRecord.putString(kStringRow, kStringLatitudeCol, value); }
  void setLongitude(String value) { dataRecord.putString(kStringRow, kStringLongitudeCol, value); }
  void setIsTowerLocation(boolean value) { dataRecord.putShort(kShortRow, kShortIsTowerLocationCol, value ? 1 : 0); }
  void setIsIndoors(boolean value) { dataRecord.putShort(kShortRow, kShortIsIndoorsCol, value ? 1 : 0); }
  void setDelete(boolean value) { dataRecord.putShort(kShortRow, kShortDeleteCol, value ? 1 : 0); }
  void setRecordUID(int value) { dataRecord.putInt(kIntRow, kIntRecordUIDCol, value); }

  int getSector() { 
    int towerDecimal = getTowerDecimal();
    return towerDecimal - (10 * (towerDecimal / 10));   
  }

  int getTowerNoSector() { 
    int towerDecimal = getTowerDecimal();
    return towerDecimal / 10;
  }

  static int cidToTowerNoSector(String cid) { 
    int towerDecimal = hexToDec(cid);
    return towerDecimal / 10;
  }

  Record() {
    super();
    dataRecord = new DataRecord(kNumRows);
    // Expand the record to full size.
    dataRecord.putString(kStringRow, kStringLongitudeCol, ""); // last string col
    dataRecord.putInt(kIntRow, kIntRecordUIDCol, 0); // last int col
    dataRecord.putShort(kShortRow, kShortTowerDecimalCol, 0); // last short col
    dataRecord.putShort(kShortRow, kShortAsuCol, 20);
    // DEBUG.p("Record() dataRecord.mData.length = " + dataRecord.mData.length);
  }

  void update() {
    Vector data = getCellInfo();
    setOPER((String)data.elementAt(0));
    setLAC((String)data.elementAt(1));
    setCID((String)data.elementAt(2));
    setSignalStrength(danger.system.Hardware.getPhoneInterface().debugGetSignalStrength());
    // setDBM(danger.system.Hardware.getPhoneInterface().getSignalStrength())); // this number is b0rked
    setASU((short)(danger.system.Hardware.getPhoneInterface().getSignalStrength_ASU()));
    // DEBUG.p("update() dataRecord.mData.length = " + dataRecord.mData.length);
  }

  static Vector getCellInfo() {
    if (danger.system.Config.SIMULATOR()) {
      Vector v = new Vector(3);
      DEBUG.p("simulator tower");
      v.addElement("123456");
      v.addElement("9876");
      String cid=null;
      switch(((int)System.currentTimeMillis()) & 0x3) {
      case 0: cid="661D"; break;
      case 1: cid="661E"; break;
      case 2: cid="661F"; break;
      }
      v.addElement(cid);
      return v;
    } else {
      return danger.system.SysApp.getCellInfo();
    }    
  }

  Record (String postRecord) {
    this();
    parsePost(postRecord);
    // DEBUG.p("Record(String) dataRecord.mData.length = " + dataRecord.mData.length + " title="+getTitle() +" post = " + getAddEditPost());
  }

  Record (byte[] dataRecord) {
    // do not call this()
    this.dataRecord = new DataRecord(dataRecord, kNumRows);
  }

  boolean sameTower(String oper, String lac, int towerNoSector) {
    return (towerNoSector == getTowerNoSector() &&
	    oper.equals(getOPER()) && 
	    lac.equals(getLAC()));
  }

  void parsePost(String postData) {
    // DEBUG.p("parsing " + postData);
    // long startTime = System.currentTimeMillis();
    StringTokenizer t = new StringTokenizer(postData, "&=", true);
    boolean isValue = false;
    String key=null;
    String value=null;
    while (t.hasMoreTokens()) {
      String token = t.nextToken();
      if (isValue) {
	if (token.equals("&")) 
	  value="";
	else
	  value = urlDecode(token);
	isValue = false;
	put(key, value);
	key = null;
	value = null;
      } else {
	if (token.equals("=")) isValue=true;
	else key = token;
      }
    }
    if (key != null) {
      put(key, value == null ? "" : value);
    }
    // long endTime = System.currentTimeMillis();
    // DEBUG.p("parse postdata " + (endTime - startTime)+"ms");
  }

  private static String urlDecode(String token) {
    if (token.indexOf("%") > -1 || token.indexOf("+") > -1)
      return StringUtils.urlDecode(token);
    return token;
  }

  void parseDialogWindow(MyDialogWindow dw) {
    String uid = dw.getStringValue(kCellInfoUID);
    if (uid != null && uid.length()>0)
      setUID(dw.getStringValue(kCellInfoUID));
    String recordUID = dw.getStringValue(kCellInfoRecordUID);
    // DEBUG.p("parseDialogWindow recordUID="+recordUID);
    if (recordUID != null && recordUID.length()>0)
      setRecordUID(StringUtils.toInt(recordUID));
    setOPER(dw.getStringValue(kCellInfoOper));
    setLAC(dw.getStringValue(kCellInfoLAC));
    setCID(dw.getStringValue(kCellInfoCID));
    setASU(((short)(StringUtils.toInt(dw.getStringValue(kCellInfoASU)))));
    // put("dbM", dw.getStringValue(kCellInfoDBM));
    setName(dw.getStringValue(kCellInfoName));
    setAddress(dw.getStringValue(kCellInfoAddress));
    setCity(dw.getStringValue(kCellInfoCity));
    setState(dw.getStringValue(kCellInfoState));
    setZip(dw.getStringValue(kCellInfoZip));
    setLatitude(dw.getStringValue(kCellInfoLatitude));
    setLongitude(dw.getStringValue(kCellInfoLongitude));
    String signalStrength = dw.getStringValue(kCellInfoSignalStrength);
    if (signalStrength != null && signalStrength.length() > 0)
      setSignalStrength(signalStrength);
    setIsTowerLocation(dw.getBooleanValue(kCellInfoIsTowerLocation));
    setIsIndoors(dw.getBooleanValue(kCellInfoIsIndoors));
  }

  String getTitle() {
    StringBuffer sb = new StringBuffer();
    if (getCID().length()>0) {
      sb.append(getCID());
      sb.append(" ");
    }
    if (getCID().length()>0) {
      sb.append(getLAC());
      sb.append(" ");
    }
    if (getAddress().length()>0) {
      sb.append(getAddress());
      sb.append(" ");
    }
    if (getName().length()>0) {
      sb.append(getName());
      sb.append(" ");
    }
    if (getCity().length()>0) {
      sb.append(getCity());
      sb.append(" ");
    }
    return sb.toString();
  }

  public String getAddEditPost() {
    return
      "oper=" + getOPER() +
      "&lac=" + getLAC() +
      "&cid=" + getCID() +
      "&name="+ StringUtils.urlEncode(getName())+
      "&address="+ StringUtils.urlEncode(getAddress())+
      "&city="+ StringUtils.urlEncode(getCity())+
      "&state="+ StringUtils.urlEncode(getState())+
      "&zip="+ StringUtils.urlEncode(getZip())+
      "&latitude="+ StringUtils.urlEncode(getLatitude())+
      "&longitude="+ StringUtils.urlEncode(getLongitude())+
      "&isTowerLocation="+ (isTowerLocation() ? "true" : "false")+
      "&isIndoors="+ (isIndoors() ? "true" : "false")+
      (getSignalStrength() == null ? "" : "&signalStrength="+ StringUtils.urlEncode(getSignalStrength()))+
      "&asu="+ getASU() +
      //"&dbM="+ dbM +
      (getUID().length()>0 ? "&uid="+getUID() : "")+
      (hasRecordUID() ? "&recordUID="+getRecordUID() : "");
  }


  private static int hexToDec(String hex) {
    int dec = 0;
    if (hex == null || hex.length() == 0) return 0;
    for (int i = 0; i < hex.length(); i++) {
      char c = hex.charAt(i);
      int v = danger.util.StringUtils.hexCharToInt(c);
      dec = (dec * 16) + v;
    }
    return dec;
  }
}
