package com.hipme.cellinfo;

import java.util.Vector;
import danger.util.StringTokenizer;
import danger.util.StringUtils;
import danger.util.DEBUG;

public class Data extends MyHash implements Resources {
  // zip?
  String getOPER() { return getString("oper"); }
  String getLAC() { return getString("lac"); }
  String getCID() { return getString("cid"); }
  String getSignalStrength() { return getString("signalStrength"); }
  String getDBM() { return getString("dbM"); }
  int getASU() { return getInt("asu"); }
  int getTowerDecimal() { return hexToDec(getCID()); }
  boolean hasUID() { return hasValue("uid"); }
  String getUID() { return getString("uid"); }
  String getName() { return getString("name"); }
  String getAddress() { return getString("address"); }
  String getCity() { return getString("city"); }
  String getState() { return getString("state"); }
  String getLatitude() { return getString("latitude"); }
  String getLongitude() { return getString("longitude"); }
  boolean isTowerLocation() { return getBoolean("isTowerLocation"); }
  boolean isIndoors() { return getBoolean("isIndoors"); }
  boolean hasRecordUID() { return hasValue("recordUID"); }
  int getRecordUID() { return getInt("recordUID"); }
  boolean isDeleted() { return getBoolean("delete"); }

  int getSector() { 
    int towerDecimal = getTowerDecimal();
    return towerDecimal - (10 * (towerDecimal / 10));   
  }
  int getTowerNoSector() { 
    int towerDecimal = getTowerDecimal();
    return towerDecimal / 10;
  }
  static int cidToTowerNoSector(String cid) { 
    int towerDecimal = hexToDec(cid);
    return towerDecimal / 10;
  }

  Data() {
  }

  void update() {
    Vector data = danger.system.SysApp.getCellInfo();
    put("oper", (String)data.elementAt(0));
    put("lac", (String)data.elementAt(1));
    put("cid", (String)data.elementAt(2));
    put("signalStrength", danger.system.Hardware.getPhoneInterface().debugGetSignalStrength());
    // put("dbM", danger.system.Hardware.getPhoneInterface().getSignalStrength()); // this number is b0rked
    put("asu", "" + danger.system.Hardware.getPhoneInterface().getSignalStrength_ASU());
  }

  Data (byte[] record) {
    this();
    String recordString = new String(record);
    parsePost(recordString);
  }

  boolean sameTower(String oper, String lac, int towerNoSector) {
    return (towerNoSector == getTowerNoSector() &&
	    oper.equals(getOPER()) && 
	    lac.equals(getLAC()));
  }

  void parsePost(String postData) {
    long startTime = System.currentTimeMillis();
    StringTokenizer t = new StringTokenizer(postData, "&=", true);
    boolean isValue = false;
    String key=null;
    String value=null;
    while (t.hasMoreTokens()) {
      String token = t.nextToken();
      if (isValue) {
	if (token.equals("&")) 
	  value="";
	else
	  value = urlDecode(token);
	isValue = false;
	put(key, value);
	key = null;
	value = null;
      } else {
	if (token.equals("=")) isValue=true;
	else key = token;
      }
    }
    if (key != null) {
      put(key, value == null ? "" : value);
    }
    long endTime = System.currentTimeMillis();
    DEBUG.p("parse postdata " + (endTime - startTime));
  }

  private static String urlDecode(String token) {
    if (token.indexOf("%") > -1 || token.indexOf("+") > -1)
      return StringUtils.urlDecode(token);
    return token;
  }

  static String parsePostValue(String inKey, String data) {
    StringTokenizer t = new StringTokenizer(data, "&=", true);
    boolean isValue = false;
    String key=null;
    while (t.hasMoreTokens()) {
      String token = t.nextToken();
      if (isValue) {
	if (inKey.equals(key)) {
	  if (token.equals("&"))
	    return "";
	  else
	    return urlDecode(token);
	}
	isValue = false;
	key = null;
      } else {
	if (token.equals("=")) 
	  isValue=true;
	else
	  key = token;
      }
    }
    if (inKey.equals(key)) 
      return "";
    return null;
  }

  void parseDialogWindow(MyDialogWindow dw) {
    String uid = dw.getStringValue(kCellInfoUID);
    if (uid != null && uid.length()>0)
      put("uid", dw.getStringValue(kCellInfoUID));
    String recordUID = dw.getStringValue(kCellInfoRecordUID);
    if (recordUID != null && recordUID.length()>0)
      put("recordUID", dw.getStringValue(kCellInfoRecordUID));
    put("oper", dw.getStringValue(kCellInfoOper));
    put("lac", dw.getStringValue(kCellInfoLAC));
    put("cid", dw.getStringValue(kCellInfoCID));
    put("asu", dw.getStringValue(kCellInfoASU));
    // put("dbM", dw.getStringValue(kCellInfoDBM));
    put("name", dw.getStringValue(kCellInfoName));
    put("address", dw.getStringValue(kCellInfoAddress));
    put("city", dw.getStringValue(kCellInfoCity));
    put("state", dw.getStringValue(kCellInfoState));
    put("latitude", dw.getStringValue(kCellInfoLatitude));
    put("longitude", dw.getStringValue(kCellInfoLongitude));
    String signalStrength = dw.getStringValue(kCellInfoSignalStrength);
    if (signalStrength != null && signalStrength.length() > 0)
      put("signalStrength", signalStrength);
    put("isTowerLocation", dw.getBooleanValue(kCellInfoIsTowerLocation) ? "true" : "false");
    put("isIndoors", dw.getBooleanValue(kCellInfoIsIndoors) ? "true" : "false");
  }

  String getTitle() {
    StringBuffer sb = new StringBuffer();
    if (hasValue("cid")) {
      sb.append(getString("cid"));
      sb.append(" ");
    }
    if (hasValue("lac")) {
      sb.append(getString("lac"));
      sb.append(" ");
    }
    if (hasValue("address")) {
      sb.append(getString("address"));
      sb.append(" ");
    }
    if (hasValue("name")) {
      sb.append(getString("name"));
      sb.append(" ");
    }
    if (hasValue("city")) {
      sb.append(getString("city"));
      sb.append(" ");
    }
    return sb.toString();
  }

  public String getAddEditPost() {
    return
      "oper=" + getOPER() +
      "&lac=" + getLAC() +
      "&cid=" + getCID() +
      "&name="+ StringUtils.urlEncode(getName())+
      "&address="+ StringUtils.urlEncode(getAddress())+
      "&city="+ StringUtils.urlEncode(getCity())+
      "&state="+ StringUtils.urlEncode(getState())+
      "&latitude="+ StringUtils.urlEncode(getLatitude())+
      "&longitude="+ StringUtils.urlEncode(getLongitude())+
      "&isTowerLocation="+ (isTowerLocation() ? "true" : "false")+
      "&isIndoors="+ (isIndoors() ? "true" : "false")+
      (getSignalStrength() == null ? "" : "&signalStrength="+ StringUtils.urlEncode(getSignalStrength()))+
      "&asu="+ getASU() +
      //"&dbM="+ dbM +
      (hasValue("uid") ? "&uid="+getUID() : "")+
      (hasValue("recordUID") ? "&recordUID="+getRecordUID() : "");
  }


  private static int hexToDec(String hex) {
    int dec = 0;
    if (hex == null || hex.length() == 0) return 0;
    for (int i = 0; i < hex.length(); i++) {
      char c = hex.charAt(i);
      int v = danger.util.StringUtils.hexCharToInt(c);
      dec = (dec * 16) + v;
    }
    return dec;
  }
}
